@@ -17,7 +17,7 @@ use Data::Table::Text qw(:all);
 use File::Copy;
 use POSIX qw(strftime);                                                         # http://www.cplusplus.com/reference/ctime/strftime/
 
-our $VERSION = '2017.432';
+our $VERSION = '2017.433';
 
 #-------------------------------------------------------------------------------
 # Constants
@@ -51,10 +51,11 @@ if (1)
   qw(device),                                                                   # Optional: Device to run on, default is the only emulator
   qw(domain),                                                                   # REQUIRED: Domain name for app
   qw(icon),                                                                     # Optional: Jpg file containing a picture that will be scaled to make an icon for the app, default is 'icon.jpg'
+  qw(keyAlias),                                                                 # REQUIRED: alias used in keytool to name the key to be used to sign this app
   qw(keyStoreFile),                                                             # REQUIRED: file name of keystore
-  qw(keyStorePwd),                                                              # Output:   message log
-  qw(log),                                                                      # REQUIRED: password of keystore
-  qw(libs),                                                                     # REQUIRED: password of keystore
+  qw(keyStorePwd),                                                              # REQUIRED: password of keystore
+  qw(log),                                                                      # Output:   message log
+  qw(libs),                                                                     # Optional: extra libraries
   qw(name),                                                                     # Optional: One word name of app, else the name of the folder: '../'
   qw(parameters),                                                               # Optional: Parameter string to be placed in res for the app
   qw(permissions),                                                              # Optional: Permissions, a standard useful set is applied
@@ -126,13 +127,10 @@ sub manifestFile($)     {my ($a) = @_; $a->buildArea.'AndroidManifest.xml'}
 
 sub logMessage($@)                                                              # Log a message
  {my ($android, @message) = @_;
-  for(@message)
-   {$_ or confess;
-   }
-  my $s = join '', @message;
+  my $s = join '', grep {$_} @message;
   chomp($s) if $s =~ /\n\Z/;
   push @{$android->log}, $s;
-  say STDERR $s;
+# say STDERR $s;
  }
 
 #-------------------------------------------------------------------------------
@@ -142,6 +140,9 @@ sub logMessage($@)
 sub pushIcon                                                    # Create and transfer each icon  using Imagemagick
  {my ($android, $size, $dir) = @_;
   my $icon    = $android->icon;
+  $icon or
+    confess "Use the icon() method to supply an icon file for this app\n";
+  -e $icon or confess "Cannot find icon file:\n$icon\n";
   my $appIcon = $android->appIcon;
   my $res     = $android->appResFolder;
   my $man     = $android->manifestFile;
@@ -149,11 +150,9 @@ sub pushIcon                                                    # Create and tra
    {for my $d(qw(drawable))
      {makePath($appIcon);
       my $s = $size;
-      my $I = $icon;
-      -e $I or confess;
-      my $c = "convert -strip $I -resize ${s}x${s}! $appIcon";
-      $android->logMessage($c);
-      $android->logMessage(qx($c));
+      my $c = "convert -strip $icon -resize ${s}x${s}! $appIcon";
+      my $r = xxx($c);
+      !$r or confess "Unable to create icon:\n$r\n";
 
       my $res = $android->appResFolder;
       my $T = $res.$d.'-'.$dir.'dpi/'.$i.'.png';
@@ -308,19 +307,23 @@ END
 
 sub create
  {my ($android) = @_;
-  my $app       = $android->buildArea;
+  my $buildArea = $android->buildArea;
   my $sdk       = $android->sdk;
   my $target    = $android->target;
   my $name      = $android->appName;
   my $activity  = $android->activity;
   my $package   = $android->package;
-  qx(rm -r $app);
-  $android->pushIcons;                                                         # Create icons
-  $android->copySource;                                                        # Copy source
-  $android->copyLibs;                                                          # Copy libraries
-  $android->antProperties;                                                     # Ant properties
-  $android->manifest;                                                          # Create manifest
-  $android->resources;                                                         # Create resources
+  if (-d $buildArea)
+   {my $r = xxx("rm -r $buildArea");                                                    # Remove existing app
+    !$r or confess "Unable to remove existing app build area:\n".
+                   "$buildArea\n$r\n";
+   }
+  $android->pushIcons;                                                          # Create icons
+  $android->copySource;                                                         # Copy source
+  $android->copyLibs;                                                           # Copy libraries
+  $android->antProperties;                                                      # Ant properties
+  $android->manifest;                                                           # Create manifest
+  $android->resources;                                                          # Create resources
  }
 
 #-------------------------------------------------------------------------------
@@ -334,9 +337,21 @@ sub make
 
   my $sdk        = $android->sdk;
   my $buildTools = $android->buildTools;
-  my $buildArea  = $android->buildArea;
+  $buildTools or confess
+   "Supply the path to the build-tools folder in your Android sdk".
+   " via the buildTools() method\n";
+
+  my $buildArea    = $android->buildArea;
   my $keyStoreFile = $android->keyStoreFile;
+     $keyStoreFile or confess
+   "Supply the path to the key store file via the keyStoreFile() method\n";
+  -e $keyStoreFile or confess
+   "Key store file does not exists:\n$keyStoreFile\n";
+  my $keyAlias     = $android->keyAlias;
+     $keyAlias or confess
+   "Supply the key alias to sign this app via the keyAlias() method\n";
   my $keyStorePwd  = $android->keyStorePwd;
+
   my $adb        = filePath($sdk, "platform-tools/adb");
   my $androidJar = $android->androidJar;
 
@@ -358,28 +373,29 @@ sub make
 
   if (1)                                                                        # Confirm aapt
    {my $a = xxx("$aapt version");
-    $a =~ /Android Asset Packaging Tool/ or confess "aapt not found at:\n$aapt";
+    $a =~ /Android Asset Packaging Tool/ or
+      confess "aapt not found at:\n$aapt\n";
    }
 
   if (1)                                                                        # Confirm javac
    {my $a = xxx("javac -version");
-    $a =~ /javac/ or confess "javac not found";
+    $a =~ /javac/ or confess "javac not found\n";
    }
 
   if (1)                                                                        # Confirm dx
    {my $a = xxx("$dx --version");
-    $a =~ /dx version/ or confess "dx not found at:\n$dx";
+    $a =~ /dx version/ or confess "dx not found at:\n$dx\n";
    }
 
   if (1)                                                                        # Confirm zipalign
    {my $a = xxx("$zipAlign");
     $a =~ /Zip alignment utility/ or
-      confess "zipalign not found at:\n$zipAlign";
+      confess "zipalign not found at:\n$zipAlign\n";
    }
 
   if (1)                                                                        # Confirm zipalign
    {my $a = xxx("$adb version");
-    $a =~ /Android Debug Bridge/ or confess "adb not found at:\n$adb";
+    $a =~ /Android Debug Bridge/ or confess "adb not found at:\n$adb\n";
    }
 
   if (1)                                                                        # Confirm files
@@ -400,8 +416,10 @@ sub make
 
   if (1)                                                                        # Generate R.java
    {makePath($gen);
-    xxx("$aapt package -f -m -0 apk -M $manifest -S $res -I $androidJar",
-        "-J $gen --generate-dependencies");
+    my $r = xxx
+    ("$aapt package -f -m -0 apk -M $manifest -S $res -I $androidJar",
+     "-J $gen --generate-dependencies");
+    $android->logMessage($r);
    }
 
   if (1)                                                                        # Java
@@ -411,24 +429,29 @@ sub make
       findFiles(filePathDir($gen));
 
     my $r = xxx("javac -source  7 -target 7 -cp $androidJar -d $classes $j");
-    $r !~ /error/ or confess "Java errors";
+    $r !~ /error/ or confess "Java errors\n";
+    $android->logMessage($r);
    }
 
   if (1)                                                                        # Dx
    {makePath($classes);
-    xxx("$dx --dex --force-jumbo --output $classes.dex $classes");
+    my $r = xxx("$dx --dex --force-jumbo --output $classes.dex $classes");
+    $android->logMessage($r);
    }
 
   if (1)                                                                        # Crunch
    {makePath($binRes);
-    xxx("$aapt crunch -v -S $res -C $binRes");
+    my $r = xxx("$aapt crunch -v -S $res -C $binRes");
+    $android->logMessage($r);
    }
 
   if (1)                                                                        # Package
-   {xxx("$aapt package --no-crunch -f  -0 apk -M $manifest",
-        "-S $binRes  -S $res -I $androidJar",
-        "-F $api",
-        "--generate-dependencies");
+   {my $r = xxx
+     ("$aapt package --no-crunch -f  -0 apk -M $manifest",
+      "-S $binRes  -S $res -I $androidJar",
+      "-F $api",
+      "--generate-dependencies");
+    $android->logMessage($r);
    }
 
   if (1)                                                                        # Create apk and sign
@@ -436,13 +459,22 @@ sub make
     xxx("cd $bin && zip -qv $apj classes.dex");                                 # Add dexed classes
 
     my $z = xxx("$zipAlign -f 4 $apj $apk");
+    $android->logMessage($z);
 
     my $s = xxx("echo $keyStorePwd |",                                          # Sign
-                "jarsigner -verbose -keystore $keyStoreFile $apk $app");
-    $s =~ /jar signed/s or confess "Unable to sign $apk";
+                "jarsigner -verbose -keystore $keyStoreFile $apk $keyAlias");
+
+    $s =~ /reference a valid KeyStore key entry containing a private key/s and
+      confess "Invalid keystore password: $keyStorePwd ".
+              "for keystore:\n$keyStoreFile\n".
+              "Specify the correct password via the keyStorePwd() method\n";
+
+    $s =~ /jar signed/s or confess "Unable to sign $apk\n";
+    $android->logMessage($s);
 
     my $v = xxx("jarsigner -verify $apk");
-    $v =~ /jar verified/s  or confess "Unable to verify $apk";
+    $v =~ /jar verified/s  or confess "Unable to verify $apk\n";
+    $android->logMessage($v);
    }
  }
 
@@ -458,7 +490,7 @@ sub lint
   my $cmd = qq(cd $src && javac *.java -cp  $androidJar:$area);                 # Android, plus locally created classes
   $android->logMessage($cmd);
   if (my $r = qx($cmd))                                                         # Perform compile
-   {confess $r;
+   {confess "$r\n";
    }
  }
 
@@ -486,11 +518,19 @@ sub install
   my $apk        = $android->apk;
   my $device     = $android->device;
   my $package    = $android->package;
-  my $adb        = $sdk."platform-tools/adb";
-  for("$adb -s $device install -r $apk",
-      "$adb -s $device shell am start $package/.Activity")
-   {$android->logMessage($_);
-    print STDERR qx($_)
+  my $activity   = $android->activity;
+  my $adb        = $sdk."platform-tools/adb -s $device";
+  if (1)
+   {my $c = "$adb install -r $apk";
+    my $r = xxx($c);
+    $r =~ /Success/ or confess "Install failed\n$r\n";
+    $android->logMessage($r);
+   }
+  if (1)
+   {my $c = "$adb shell am start $package/.Activity";
+    my $r = xxx($c);
+    $r =~ /Starting/ or confess "Start failed\n$r\n";
+    $android->logMessage($r);
    }
  }
 
@@ -500,7 +540,6 @@ sub install
 
 sub cInstall                                                                    # Install on emulator
  {my ($android)  = @_;
-  $android->logMessage("Install");
   $android->install;
  }
 
@@ -537,7 +576,7 @@ sub build
     elsif (/\A-*lint\z/i)    {$android->cLint}                                  # Lint source
     elsif (/\A-*install\z/i) {$android->cInstall}                               # Install on emulator
     else
-     {confess"Ignored unknown command: $_";
+     {confess"Ignored unknown command: $_\n";
      }
    }
   $android->logMessage("Normal finish for Android::Build");
